	SUBROUTINE GDFSPC  ( dspc, ibnd, ilat1, ilon1, ilat2, ilon2,
     +			     dphi, maxab, rnav, rgrd, rmean, rvar,
     +			     az, bz, lnum, phibnd, iret )
C************************************************************************
C* GDFSPC								*
C*									*
C* This subroutine creates a zonal band and does a spectral analysis	*
C* of meridionally averaged grid values.				*
C*									*
C* GDFSPC  ( DSPC, IBND, ILAT1, ILON1, ILAT2, ILON2, DPHI, MAXAB,	*
C*	     RNAV, RGRD, RMEAN, RVAR, AZ, BZ, LNUM, PHIBND, IRET )	*
C*									*
C* Input parameters:							*
C*	DSPC		REAL		Resolution (deg) of zonal band	*
C*	IBND		INTEGER		Zonal band number		*
C*	ILAT1		INTEGER		SW latitude bound of band # 1	*
C*	ILON1		INTEGER		SW longitude bound of band # 1	*
C*	ILAT2		INTEGER		NE latitude bound of last band	*
C*	ILON2		INTEGER		NE longitude bound of last band	*
C*	DPHI		REAL		Zonal band width		*
C*	MAXAB		INTEGER		Max # of spectral components	*
C*	RNAV (*)	REAL		Grid navigation block		*
C*	RGRD (*)	REAL		Grid of data			*
C*									*
C* Output parameters:							*
C*	RMEAN		REAL		Mean value of zonal band	*
C*	RVAR		REAL		Variance of zonal band		*
C*	AZ (LNUM)	REAL*8		COSINE function coefficients	*
C*	BZ (LNUM)	REAL*8		SINE function coefficients	*
C*	LNUM		INTEGER		Number of coefficients		*
C*	PHIBND		REAL		Center latitude of the band	*
C*	IRET		INTEGER		Return code			*
C*					  0 = normal return		*
C*					 -8 = zonal subgrid is too big	*
C*					 -9 = cannot set grd navigation *
C*					-10 = GTRANS failed		*
C*					-11 = Horiz. interp. failed	*
C*					-12 = meridional colmn missing  * 
C**									*
C* Log:									*
C* K. Brill/HPC          4/00						*
C* K. Brill/HPC		 7/00	Double precision arrays			*
C************************************************************************
	INCLUDE		'GEMPRM.PRM'
	PARAMETER	( MXC = LLMXGD )
C*
	REAL		rnav (*), rgrd (*)
	DOUBLE
     +	PRECISION	az (*), bz (*)
C*
	REAL		zbnd (MXC), zx (MXC), zy (MXC)
C*
	INCLUDE		'ERMISS.FNC'
C------------------------------------------------------------------------
	iret = 0
C
C*	Compute dimensions of zone grid.
C
	rlats = FLOAT ( ilat1 ) + dphi * FLOAT ( ibnd - 1 )
	rlatn = FLOAT ( ilat1 ) + dphi * FLOAT ( ibnd )
	rlonw = FLOAT ( ilon1 )
	rlone = FLOAT ( ilon2 )
	nx = NINT ( ( rlone - rlonw ) / dspc ) + 1
	ny = NINT ( ( rlatn - rlats ) / dspc ) + 1
	IF ( nx * ny .gt. MXC ) THEN
	    iret = -8
	    RETURN
	END IF
C
C*	Compute lat/lon of every point on the zone grid.
C
	ii = 0
	DO j = 1, ny
	    rlat = rlats + FLOAT (j-1) * dspc
	    DO i = 1, nx
		ii = ii + 1
		zx (ii) = rlat
		zy (ii) = rlonw + FLOAT (i-1) * dspc
	    END DO
	END DO
	nxny = ii
C
C*	Transform the lat/lon to grid relative positions.
C
	CALL GR_SNAV ( 20, rnav, ier )
	IF ( ier .ne. 0 ) THEN
	    CALL ER_WMSG ( 'GR', ier, ' ', ierr )
	    iret = -9
	    RETURN
	END IF
	CALL GTRANS ( 'M', 'G', nxny, zx, zy, zx, zy, ier )
	IF ( ier .ne. 0 ) THEN
	    iret = -10
	    RETURN
	END IF
C
C*	Interpolate to zonal band grid.
C
	kx = NINT ( rnav (5) )
	ky = NINT ( rnav (6) )
	CALL GR_INTP ( 0, zx, zy, nxny, kx, ky, rgrd, zbnd, ier )
	IF ( ier .ne. 0 ) THEN
	    CALL ER_WMSG ( 'GR', ier, ' ', ierr )
	    iret = -11
	    RETURN
	END IF
C
C*	Reduce to one-dimension by meridional averaging.
C
	DO i = 1, nx
	    iavg = 0
	    avg = 0.0
	    DO j = i, nxny-nx+i, nx
		IF ( .not. ERMISS ( zbnd (j) ) ) THEN
		    iavg = iavg + 1
		    avg = avg + zbnd (j)
		END IF
	    END DO
	    IF ( iavg .eq. 0 ) THEN
		iret = -12
		RETURN
	    ELSE
		zbnd (i) = avg / FLOAT (iavg)
	    END IF
	END DO
	CALL FOURIER ( zbnd, nx, rmean, rvar, az, bz, iret )
	lnum = nx / 2
	phibnd = ( rlats + rlatn ) / 2.
C*
	RETURN
	END
