	SUBROUTINE MKCRPS (nrecs, nrgn, orecs, anlyg, parms, np, 
     +                     ftime, fclvl, fcvcd, ipoly, wgrd, kxy,
     +			   nlun, luns, ijskip, frcpv, iret )
C************************************************************************
C* MKCRPS								*
C*									*
C* This subroutine computes CRPS record values and writes them out in	*
C* VSDB	format.								*
C*									*
C* IPOLY is an array of integer identifiers for subregions.  If NRECS   *
C* exceeds nrgn, the entire domain (IPOLY(i)>0) is the last record,     *
C* so that nrgn+1 records are written.  The index into the ORECS array  *
C* of VSDB headers appears in IPOLY at every grid point inside the      *
C* corresponding subregion.                                             *
C*									*
C*									*
C* MKCRPS ( NRECS, NRGN, ORECS, ANLYG, PARMS, NP, FTIME, FCLVL,         *
C*          FCVCD, IPOLY WGRD, KXY, NLUN, LUNS, FRCPV, IRET )   	*
C*									*
C* Input parameters:							*
C*      NRECS           INTEGER         Number of VSDB records to make  *
C*      NRGN            INTEGER         Number of regions               *
C*	ORECS (*)	CHAR*		VSDB header records		*
C*	ANLYG (KXY)	REAL		Analysis grid			*
C*	PARMS (NP)      CHAR*		Percentile value GFUNC names    *
C*      NP              INTEGER         Number GFUNC names              *
C*      FTIME           CHAR*           Percentile grid GDATTIM stamp   *
C*      FCLVL           CHAR*           Percentile grid GLEVEL stamp    *
C*      FCVCD           CHAR*           Percentile grid GVCORD stamp    *
C*	IPOLY (KXY)	INTEGER		Integer subregion identifiers   *
C*	WGRD  (KXY)	REAL		Grid box area weights           *
C*	KXY		INTEGER		Number of grid points		*
C*	NLUN		INTEGER		Number of output units		*
C*	LUNS (NLUN)	INTEGER		Output unit numbers		*
C*      IJSKIP          INTEGER         Number of points to skip to     *
C*                                      speed up uncertainty calc       *
C*									*
C* Input and output parameter:						*
C*	FRCPV (KXY)	REAL		Current percentile level value  *
C*                                      grid (comes in as first, sub-   *
C*                                      sequent levels are read herein) *
C*									*
C* Output parameters:							*
C*	IRET		INTEGER		Return code			*
C*					  0 = normal return		*
C*                                      -23 = DG_GRID error             *
C**									*
C* Log:									*
C* K. Brill/IMSG      20160616  Created					*
C* K. Brill/IMSG      20160630  Add ircnt                               *
C* K. Brill/IMSG      20160725  Set MXNP1 = 12; set MXD = 28            *
C* K. Brill/IMSG      20160727  Format for up to 28 values per record   *
C* K. Brill/IMSG      20170223  Use double precision for CRPS output    *
C* K. Brill/IMSg      20170316  Add IJSKIP                              *
C************************************************************************
	INCLUDE		'GEMPRM.PRM'
	PARAMETER	( MXC=512, MXNP1=12, MXREC=20, MXD=28 )
C*
	REAL 		anlyg (*), frcpv (*), wgrd (*)
	INTEGER		ipoly (*), luns (*)
	CHARACTER*(*)	orecs(*), parms(*)
        CHARACTER*(*)   ftime, fclvl, fcvcd
	
C**********
C                      CRPS STATISTIC TYPE
C
C     DATA                       CONTENTS
C     FIELD
C       1           The number of verification events (M)
C       2 -> N+2    N+1 Hersbach alpha means indexed 0 to N [26,27]
C       N+3 -> 2N+3 N+1 Hersbach beta means indexed 0 to N [26,27]
C       2N + 4      Mean fraction of outliers below first percentile [33]
C       2N + 5      Mean fraction of outliers above last percentile [33]
C       2N + 6      Mean uncertainty [19] =
C                   (1/M) Sum_k=2->M [ (1/M) Sum_l=1->k-1 |A_k - A_l| ],
C                   where A an analysis (observed) value.
C
C    N = np is the number of forecast percentile levels.
C
C    Numbers in square brackets [] are equation numbers from Hersbach
C    (2000).
C
C    Reference:
C
C    Hersbach, H., 2000:  Decomposition of the continuous ranked
C         probability score for ensemble prediction systems.  WEA.
C         FORECASTING, 15, 559-570.
C
C**********
C
	LOGICAL		found
	CHARACTER	cl1l2*(MXC), pfunc*128, atime(2)*20, parm*12
        CHARACTER       orec*128, gfunc*128
        INTEGER         level (2)
C*	FRCP1  (KXY)	REAL		Working array storing next      *
C*                                      percentile level grid           *
        REAL            frcp1 (LLMXGD)
        REAL            galfa (LLMXGD), gbeta (LLMXGD), gtmp (LLMXGD)
        REAL*8          alfbar (MXREC,MXNP1), betbar (MXREC,MXNP1) 
        REAL*8          o0bar (MXREC), onbar (MXREC), drec (MXD)
        REAL*8          otmp (MXREC), wsum (MXREC), ucbar (MXREC)
C*
	INCLUDE		'ERMISS.FNC'
C------------------------------------------------------------------------
	iret = 0
C
C*      First percentile level is already in frcpv.  Start by processing
C*      left outlier interval nn=0.
C
        indx = 1
        DO i = 1, kxy
	    IF ( .not. ERMISS ( anlyg (i) ) .and.
     +		 .not. ERMISS ( frcpv (i) ) ) THEN
C
C*              Note:  Exact equality is not an outlier.  This is a
C*                     deviation from Hersbach (2000).
C
                IF ( anlyg (i) .lt. frcpv (i) ) THEN
                    galfa (i) = 0
                    gbeta (i) = frcpv (i) - anlyg (i)
                    gtmp (i) = 1
                ELSE
                    galfa (i) = 0
                    gbeta (i) = 0
                    gtmp (i) = 0
                END IF
            ELSE
                galfa (i) = RMISSD
                gbeta (i) = RMISSD
                gtmp (i) = RMISSD
            END IF
        END DO
        CALL CMPAVG ( nrecs, nrgn, wgrd, galfa, ipoly, kxy, otmp, wsum,
     +                iret )
        DO j = 1, nrecs
            alfbar (j,indx) = otmp (j)
        END DO
        CALL CMPAVG ( nrecs, nrgn, wgrd, gbeta, ipoly, kxy, otmp, wsum,
     +                iret )
        DO j = 1, nrecs
            betbar (j,indx) = otmp (j)
        END DO
        CALL CMPAVG ( nrecs, nrgn, wgrd, gtmp, ipoly, kxy, o0bar, wsum,
     +                iret )
C
C*      Loop over the 1 to NP-1 interior intervals.
C
        ircnt = 0
        nn = 0
        npm1 = np - 1
	DO WHILE ( nn .lt. npm1 )
            nn = nn + 1
            np1 = nn + 1
            indx = indx + 1
C
C*          Read next percentile level in frcp1:
C
            gfunc = parms (np1)
            CALL ST_LSTR ( gfunc, ls1, ier )
            CALL ST_LSTR ( ftime, ls2, ier )
            CALL ST_LSTR ( fclvl, ls3, ier )
            CALL ST_LSTR ( fcvcd, ls4, ier )
            gfunc = gfunc (1:ls1) // '^' // ftime (1:ls2) // '@' //
     +              fclvl (1:ls3) // '%' // fcvcd (1:ls4) // '+2'
            CALL DG_GRID ( ftime, fclvl, fcvcd, gfunc,
     +                     pfunc, frcp1, mx, my, atime, level,
     +                     ivcord, parm, ierr )
            IF ( ierr .ne. 0 ) THEN
                CALL ER_WMSG ( 'DG', ierr, ' ', ier )
                iret = -23
                RETURN
            ELSE
                DO i = 1, kxy
	            IF ( .not. ERMISS ( anlyg (i) ) .and.
     +		         .not. ERMISS ( frcp1 (i) ) .and.
     +		         .not. ERMISS ( frcpv (i) ) ) THEN
C
C*                      Correct error in order statistics using equality.
C
                        IF ( frcp1 (i) .lt. frcpv (i) ) THEN
                            frcp1 (i) = frcpv (i)
                            ircnt = ircnt + 1
                        END IF 
C*
                        IF ( anlyg (i) .ge. frcp1 (i) ) THEN
                            galfa (i) = frcp1 (i) - frcpv (i)
                            gbeta (i) = 0
                        ELSE IF ( anlyg (i) .ge. frcpv (i) .and.
     +                            anlyg (i) .lt. frcp1 (i) ) THEN
                            galfa (i) = anlyg (i) - frcpv (i)
                            gbeta (i) = frcp1 (i) - anlyg (i)
                        ELSE
                            galfa (i) = 0
                            gbeta (i) = frcp1 (i) - frcpv (i)
                        END IF
                    ELSE
                        galfa (i) = RMISSD
                        gbeta (i) = RMISSD
                    END IF
                END DO
                CALL CMPAVG ( nrecs, nrgn, wgrd, galfa, ipoly, kxy,
     +                        otmp, wsum, iret )
                DO j = 1, nrecs
                    alfbar (j,indx) = otmp (j)
                END DO
                CALL CMPAVG ( nrecs, nrgn, wgrd, gbeta, ipoly, kxy,
     +                        otmp, wsum, iret )
                DO j = 1, nrecs
                    betbar (j,indx) = otmp (j)
                END DO
C
C*              Move current level to frcpv:
C
                DO i = 1, kxy
                    frcpv (i) = frcp1 (i)
                END DO
            END IF
        END DO
        IF ( ircnt .gt. 0 ) THEN
            WRITE (6,*) ' Warning:  ', ircnt,
     +          ' order statistics corrections were made.'
        END IF
C
C*      Last percentile level is already in frcp1.  Finish by processing
C*      right outlier interval nn=np+1.
C
        indx = indx + 1
        DO i = 1, kxy
	    IF ( .not. ERMISS ( anlyg (i) ) .and.
     +		 .not. ERMISS ( frcp1 (i) ) ) THEN
                IF ( anlyg (i) .gt. frcp1 (i) ) THEN
                    galfa (i) = anlyg (i) - frcp1 (i)
                    gbeta (i) = 0
                    gtmp (i) = 0
                ELSE
                    galfa (i) = 0
                    gbeta (i) = 0
                    gtmp (i) = 1
                END IF
            ELSE
                galfa (i) = RMISSD
                gbeta (i) = RMISSD
                gtmp (i) = RMISSD
            END IF
        END DO
        CALL CMPAVG ( nrecs, nrgn, wgrd, galfa, ipoly, kxy, otmp, wsum,
     +                iret )
        DO j = 1, nrecs
            alfbar (j,indx) = otmp (j)
        END DO
        CALL CMPAVG ( nrecs, nrgn, wgrd, gbeta, ipoly, kxy, otmp, wsum,
     +                iret )
        DO j = 1, nrecs
            betbar (j,indx) = otmp (j)
        END DO
        CALL CMPAVG ( nrecs, nrgn, wgrd, gtmp, ipoly, kxy, onbar, wsum,
     +                iret )
C
C*      Compute the uncertainty terms and check NPTS array values.
C
        CALL CMPUNC ( nrecs, nrgn, wgrd, galfa, anlyg, ipoly,
     +                kxy, ijskip, ucbar, iret )
        IF ( iret .ne. 0 ) THEN
            RETURN
        END IF
C
C*      Construct and write out NRECS number of VSDB records.
C
        DO irec = 1, nrecs
            rn = wsum (irec)
            IF ( rn .gt. 0 ) THEN
                drec (1) = rn
                jj = 1
                DO i = 1, indx
                    jj = jj + 1
                    drec (jj) = alfbar (irec,i)
                END DO 
                DO i = 1, indx
                    jj = jj + 1
                    drec (jj) = betbar (irec,i)
                END DO 
                jj = jj + 1
                drec (jj) = o0bar (irec)
                jj = jj + 1
                drec (jj) = onbar (irec)
                jj = jj + 1
                drec (jj) = ucbar (irec)
	        WRITE (cl1l2,1000) (drec(k),k=1,jj)
1000	        FORMAT ( 28(1X,1PD13.6) )
	        CALL ST_RXBL ( cl1l2, cl1l2, lnch2, ier )
                orec = orecs (irec)
                CALL ST_LSTR ( orec, lnch, ier )
	        DO i = 1, nlun
		    WRITE (luns(i), '(A,A,A)') orec (1:lnch),
     +		    		' = ', cl1l2 (1:lnch2)
	        END DO
	    ELSE
                orec = orecs (irec)
                CALL ST_LSTR ( orec, lnch, ier )
	        WRITE (6,*) orec(1:lnch), ' --------:'
	        WRITE (6,*)
     +		' All points zero, missing, or not in region.'
	    END IF
        END DO
C*
	RETURN
	END
