	SUBROUTINE GDLFLT ( nwl, grdi, kxd, kyd, grdo, iret )
C************************************************************************
C* GDLFLT								*
C*									*
C* This subroutine smoothes a scalar grid using a moving average	*
C* low-pass filter whose weights are determined by the normal 		*
C* (Gaussian) probability distribution function for two dimensions.	*
C* The weight given to any grid point within the area covered by the	*
C* moving average for a	target grid point is proportional to		*
C*									*
C*		    EXP [ -( D ** 2 ) ],				*
C*									*
C* where D is the distance from	that point to the target point divided	*
C* by the standard deviation of the normal distribution.  The value of	*
C* the standard deviation is determined by the degree of filtering	*
C* requested.  The degree of filtering is specified by an integer.	*
C* This integer is the number of grid increments from crest to crest	*
C* of the wave for which the theoretical response is 1/e = .3679.  If 	*
C* the grid increment is called delta_x, and the value of this integer	*
C* is represented by N, then the theoretical filter response function 	*
C* value for the N * delta_x wave will be 1/e.  The actual response 	*  
C* function will be greater than the theoretical value.			*
C*									*
C* The larger N is, the more severe the filtering will be, because the	*
C* response function for all wavelengths shorter than N * delta_x	*
C* will be less than 1/e.  Furthermore, as N is increased, the slope	*
C* of the filter response function becomes more shallow; so, the	*
C* response at all wavelengths decreases, but the amount of decrease	*
C* lessens with increasing wavelength.  (The theoretical response	*
C* function can be obtained easily--it is the Fourier transform of the	*
C* weight function described above.)					*
C*									*
C* The area of the patch covered by the moving average varies with N.	*
C* As N gets bigger, the smoothing gets stronger, and weight values	*
C* farther from the target grid point are larger because the standard	*
C* deviation of the normal distribution is bigger.  Thus, increasing	*
C* N has the effect of expanding the moving average window as well as	*
C* changing the values of weights.  The patch is a square covering all	*
C* points whose weight values are within two standard deviations of the	*
C* mean of the two dimensional normal distribution.  The leftover	*
C* weight values representing the fringe of the distribution are	*
C* applied to the target grid point.  This has the effect of increasing	*
C* the response function over the theoretical value.			*
C*									*
C* When this function is invoked, the first argument is the value of N	*
C* as described above, where N > 1.  If N <= 1, N = 2 is assumed.  For	*
C* example, if N = 4, then the 4 delta x wave length is passed with	*
C* approximate response	1/e.						*
C*									*
C* GRDI and GRDO must be different arrays.				*
C*									*
C* GDLFLT  ( NWL, GRDI, KXD, KYD, GRDO, IRET )				*
C*									*
C* Input parameters:							*
C*	NWL		INTEGER		Wave length in DX units		*
C*	GRDI (KXD,KYD)	REAL		Input grid			*
C*	KXD		INTEGER		X dimension of GRDI		*
C*	KYD		INTEGER		Y dimension of GRDI		*
C* Output parameters:							*
C*	GRDO (KXD,KYD)	REAL		Output grid			*
C*	IRET		INTEGER		Return code			*
C*									*
C**									*
C* Log:									*
C* K. Brill/HPC		 3/01	Created from DF_GWFS			*
C************************************************************************
	INCLUDE		'GEMPRM.PRM'
	PARAMETER	( MAXWTS = 30 )
C*
	REAL		grdi (*), grdo (*)
C*
	REAL		w (MAXWTS, MAXWTS)
C*
        INCLUDE         'ERMISS.FNC'
C------------------------------------------------------------------------
	iret = 0
	IF ( nwl .le. 1 ) nwl = 2
C
C*	Compute the array of weights.
C
C*	The range of the filter is twice the standard deviation of the
C*	required Gaussian distribution.
C
	sgma = FLOAT ( nwl ) / ( PI * SQRT ( 2.0 ) )
	nr = NINT ( 2. * sgma )
	IF ( nr .lt. 1 ) nr = 1
	IF ( nr .ge. MAXWTS ) nr = MAXWTS - 1
C
C*	Compute the matrix of weights for one quadrant using symmetry
C*	of two dimensional Gaussian surface.
C
	sumw = 0.0
	sig2 = sgma * sgma
	aa = 1. / ( sig2 * PI )
	DO jw = 1, nr + 1
	    IF ( jw .eq. 1 ) THEN
		is = 2
	    ELSE
		is = jw
	    END IF
	    DO iw = is, nr + 1
		x = iw - 1
		y = jw - 1
		w (iw,jw) = aa * EXP ( - ( x*x + y*y ) / sig2 )
		w (jw,iw) = w (iw,jw)
		IF ( jw .eq. 1 .or. jw .eq. iw ) THEN
		    sumw = sumw + w (iw,jw)
		ELSE
		    sumw = sumw + 2. * w (iw,jw)
		END IF
	    END DO
	END DO
	sumw = 4. * sumw
	w (1,1) = 1. - sumw
C
C*	Apply weights over non-missing values.
C
	DO jj = 1, kyd
	    DO ii = 1, kxd
		is = ii - nr
		ie = ii + nr
		js = jj - nr
		je = jj + nr
		sumw = 0.0
		sumf = 0.0
		DO j = js, je
		    IF ( j .ge. 1 .and. j .le. kyd ) THEN
			DO i = is, ie
			    IF ( i .ge. 1 .and. i .le. kxd ) THEN
				iw = IABS (i-ii) + 1
				jw = IABS (j-jj) + 1
				indx = (j-1)*kxd + i
				IF ( .not. ERMISS ( grdi ( indx ) )
     +				   ) THEN
				    sumw = sumw + w (iw,jw)
				    sumf = sumf + grdi ( indx ) *
     +					   w (iw,jw)
				END IF
			    END IF
			END DO
		    END IF
		END DO
		indx = ( jj - 1 ) * kxd + ii
		IF ( sumw .ne. 0.0 .and. .not. ERMISS ( grdi ( indx ) )
     +		   ) THEN
		    grdo ( indx ) = sumf / sumw
		ELSE
		    grdo ( indx ) = RMISSD
		END IF
	    END DO
	END DO
C*
	RETURN
	END
