	PROGRAM MKVSDB
C************************************************************************
C*  This program searches a set of files for the data traces defined	*
C*  in the input control file.  It writes out VSDB records after data	*
C*  combination by date-time according to the control file instructions.*
C*									*
C*  Some important hints for users of this program:			*
C*									*
C*  1.  Set search conditions using fvs u.  Select the time series	*
C*      option, and then use P when setting the independent variable	*
C*	condition to enter the period designator.  Do not enter a range	*
C*      of date-times.							*
C*									*
C*  2.  For a fair comparison of models, models that are frequently	*
C*      compared should be done in one search with consistency		*
C*      checking activated.  This requires that you overlay more	*
C*      than one trace and select option 2 for both consistency check	*
C*      menus.  Records for all models (up to 16 allowed) will be in	*
C*      the one output file (vsdb.dat).  Use grep to separated them	*
C*      into files having names of the form mdl_YYYYMM.vsdb, where mdl	*
C*      is the name of a model, YYYY is the year, and MM is the	month.  *
C*      Each such file should then be placed in its own directory	*
C*      named mdl.							*
C*									*
C*  3.  Do not wild card any search condition, and make sure that a	*
C*      search condition exists for all 8 nontrivial VSDB header	*
C*      fields (the first field "V01" is trivial).			*
C*									*
C*  4.  Include a search condition for HHNN (valid hour-minute).  This	*
C*      information will be placed in the valid time stamp of the	*
C*      output records and allows separation of forecasts verifying	*
C*      at different times of day.					*
C*									*
C*  Without prompts, this program reads the following from standard	*
C*  input:								*
C*									*
C*  LINE			CONTENTS				*
C*   1	  name of the control file					*
C*   2	  name of a file containing the list of file names to search	*
C*									*
C*  The output is in a file named vsdb.dat.				*
C*									*
C*  The control file contains records written by the script user_in.	*
C*  The control file contains blocks of records describing search	*
C*  conditions for the data.  Such a block begins with a header record	*
C*  having the following values separated by spaces:			*
C*									*
C*	ITRC	=	Trace number					*
C*	IDDFLD	=	Header/data field for condition			*
C*	IVRTYP	=	Variable type (independent vs dependent)	*
C*	ICMTYP	=	Data combination code				*
C*	NCN	=	Number records following this header--must be 1	*
C*	IPLTNM	=	Number of this plot (used for data combination)	*
C*	IPLTYP	=	Type of plot (not used for searching)		*
C*	KMATCH  =	data loss tolerance percentage * 100 +		*
C*			  data combination consistency code		*
C*									*
C*  The remaining record in the block contains the following:		*
C*									*
C*	ICNTYP	=	Type of search condition			*
C*	R1	=	Real search value or lower bound of range	*
C*	R2	=	Real upper bound of range			*
C*	C1	=	Character search string or lower bound of range	*
C*	C2	=	Character upper bound of range			*
C*									*
C*									*
C*  The maximum number of file names allowed in the input list is 1024.	*
C*  The maximum number of points in a trace is 1024.			*
C*									*
C*  The output file (vsdb.dat) is a VSDB file based on data combined	*
C*  by time (e.g., daily records combined into monthly records).	*
C*									*
C*  Log:								*
C*  K. Brill/HPC	02/03						*
C*  K. Brill/HPC	 3/04	Change MXDPH from 12 to 16; change	*
C*				MXTRC from 16 to 24			*
C*  K. Brill/HPC	 8/05	Increase MXDPH to 24; inrec*512		*
C*  K. Brill/HPC	 8/16	Increase MXDPH to 28                    *
C************************************************************************
	PARAMETER	( NFLS = 512, NCNMX = 64, NPTMX = 2048 )
	PARAMETER	( MXDPH = 28, MXTRC = 24 )
	CHARACTER*512	filnam (NFLS)
	CHARACTER*24	charcn (2, NCNMX), clbls (NPTMX)
	CHARACTER*1	srtyp, crtype
	INTEGER		icntyp (NCNMX), mknts (NPTMX), mkndx (MXDPH)
	INTEGER		msrti (NPTMX)
	REAL		realcn (2, NCNMX), rlbls (NPTMX)
	REAL*8		trace (NPTMX, MXDPH)
	REAL*8		trcout (MXDPH)
	CHARACTER	inrec*512
	CHARACTER*32	crec (5)
C*
	CHARACTER*16	ohdr (MXTRC,9)
	CHARACTER*4	chm, chhmm
	INTEGER		lhdr (9)
C*
	INTEGER		ihdr (8)
	CHARACTER*16	chdr (8)
	REAL		rrec (3)
	LOGICAL		quiet, frhm
	DATA		quiet /.true./
C------------------------------------------------------------------------
	luno = 12
	luni = 11
	lunl = 10
C
C*	Open the output file.
C
	OPEN ( UNIT=luno, FILE='vsdb.dat', STATUS='unknown',
     +	       IOSTAT = ios )
	IF ( ios .ne. 0 ) THEN
	    WRITE (6,*) ' Cannot open the output file.'
	    STOP
	END IF
C
C*	Read control file name and open the file.
C
	READ (5,'(A)') inrec
	OPEN ( UNIT=luni, FILE=inrec, STATUS='old', IOSTAT=ios )
	IF ( ios .ne. 0 ) THEN
	    WRITE (6,*) 'Could not open control file ', inrec
	    STOP
	END IF
C
C*	Read the file name list file and open it.
C
	READ (5,'(A)') inrec
	OPEN ( UNIT=lunl, FILE=inrec, STATUS='old', IOSTAT=ios )
	IF ( ios .ne. 0 ) THEN
	    WRITE (6,*) 'Could not open list file ', inrec
	    STOP
	END IF
C
C*	Initialize search conditions.
C
	CALL SRCH_INI ( iret )
	IF ( iret .ne. 0 ) THEN
	    CALL SRCH_ERR ( iret, 'SRCH_INI', ier )
	    STOP
	END IF
C
C*	Initialize all headers to ALL.
C
	DO it = 1, MXTRC
	    ohdr (it,1) = 'V01'
	END DO
	DO ih = 2, 9
	    DO it = 1, MXTRC
		ohdr (it,ih) = 'ALL'
	    END DO
	END DO
C
C*	Now read through the control file and set conditions.
C
	chhmm = ' '
	frhm = .true.
	ios = 0
	ntrc = 0
	DO WHILE ( ios .eq. 0 )
	    READ (luni,'(A)',IOSTAT=ios) inrec
	    IF ( ios .eq. 0 ) THEN
		CALL ST_LSTR ( inrec, lng, ier )
		CALL ST_CLST ( inrec, ' ', ' ', 8, chdr, num, ier )
		IF ( ier .ne. 0 .or. num .ne. 8 ) THEN
		    WRITE (6,*) ' Invalid control file header record.'
		    STOP
		END IF
		CALL ST_ILST ( inrec, ' ', -99, 7, ihdr, num, ier )
		IF ( ier .lt. 0 .or. num .ne. 7 ) THEN
		    WRITE (6,*) ' Invalid control file header record.'
		    STOP
		END IF
		itrc = ihdr (1)
		IF ( itrc .gt. MXTRC ) THEN
		    WRITE (6,*) 'Too many traces in search conditions.'
		    STOP
		END IF
		iddfld = ihdr (2)
		IF ( iddfld .eq. 1 ) THEN
		    ioh = 1
		ELSE IF ( iddfld .eq. 2 ) THEN
		    ioh = 2
		ELSE IF ( iddfld .le. 4 ) THEN
		    ioh = 3
		ELSE IF ( iddfld .le. 7 ) THEN
		    ioh = 4
		ELSE IF ( iddfld .eq. 8 ) THEN
		    ioh = 5
		ELSE IF ( iddfld .eq. 9 ) THEN
		    ioh = 6
		ELSE IF ( iddfld .eq. 10 ) THEN
		    ioh = 7
		ELSE IF ( iddfld .eq. 11 ) THEN
		    WRITE (6,*)
     +		    'Statistic type and threshold must be given ',
     +              'together.'
		    STOP
		ELSE IF ( iddfld .eq. 12 ) THEN
		    ioh = 8
		ELSE IF ( iddfld .eq. 13 ) THEN
		    WRITE (6,*)
     +		    'Level ID and value must be given together.'
		    STOP
		ELSE IF ( iddfld .eq. 14 ) THEN
		    ioh = 9
		ELSE IF ( iddfld .le. 16 ) THEN
		    ioh = 4
		ELSE
		    WRITE (6,*) 'Search conditions on values not ',
     +				'allowed.'
		    STOP
		END IF
		ivrtyp = ihdr (3)
		IF ( ivrtyp .eq. 1 ) THEN
		    IF ( iddfld .ne. 5 ) THEN
			WRITE (6,*)
     +                  'Data combination must be over time.'
			STOP
		    END IF
		END IF
		icmtyp = ihdr (4)
		ncn = ihdr (5)
		IF ( ncn .ne. 1 ) THEN
		    WRITE (6,*)
     +		    'Only one search condition allowed for each ',
     +		    'header field.'
		    STOP
		END IF
		idplot = ihdr (6)
		islsh = INDEX ( chdr (8), '/' )
		IF ( islsh .eq. 0 ) THEN
C
C*		    Backward compatibility code.
C
		    CALL ST_NUMB ( chdr (8), kmatch, ier )
		    kmch = MOD ( kmatch, 100 )
		    kase = 0
		    kpdz = kmch / 10
		    kltp = kmatch / 100
		    kmch = MOD ( kmch, 10 )
C
C*		    If the loss tolerance percentage is missing, then
C*		    use the old default of 33%.
C
		    IF ( kltp .eq. 0 ) THEN
			CALL ST_LSTR ( chdr (8), iln, ier )
			IF ( iln .le. 2 ) THEN
			    kltp = 33
			END IF
		    END IF
		ELSE
		    CALL ST_ILST ( chdr (8), '/', 0, 4, ihdr, num, ier )
		    kmch = ihdr (1)
		    kase = ihdr (2)
		    kpdz = ihdr (3)
		    kltp = ihdr (4)
		END IF
		ntrc = MAX ( itrc, ntrc )
		DO i = 1, ncn
		    READ ( luni,'(A)',IOSTAT=ios) inrec
		    IF ( ios .ne. 0 ) THEN
			WRITE (6,*) ' Unexpected end of control file.'
			STOP
		    END IF
		    CALL ST_LSTR ( inrec, lng, ier )
		    CALL ST_RLST ( inrec, ' ', 0, 3, rrec, num, ier )
		    IF ( ier .lt. 0 .or. num .ne. 3 ) THEN
			WRITE (6,*) ' Invalid control file condition ',
     +				    'record.'
			STOP
		    END IF
		    icntyp (i) = NINT ( rrec (1) )
		    realcn (1,i) = rrec (2)
		    realcn (2,i) = rrec (3)
C*
		    CALL ST_CLST ( inrec, ' ', ' ', 5, crec, num, ier )
		    IF ( ier .ne. 0 .or. num .ne. 5 ) THEN
			WRITE (6,*) ' Invalid control file condition ',
     +				    'record.'
			STOP
		    END IF
		    charcn (1,i) = crec (4)
		    charcn (2,i) = crec (5)
		    IF ( iddfld .eq. 15 ) THEN
			chm = crec (4)
		    ELSE IF ( iddfld .eq. 15 ) THEN
			chm = ' '
		    END IF
		    IF ( frhm .and. iddfld .eq. 15 ) THEN
			chhmm = chm
			frhm = .not. frhm
		    ELSE IF ( iddfld .eq. 15 ) THEN
			IF ( chm .ne. chhmm ) THEN
			    WRITE (6,*)
     +			    'All conditions must have same HHNN.'
			    STOP
			END IF
		    END IF
		    k = 1
		    DO WHILE ( k .lt. 5 )
			k = k + 1
			IF ( crec (k) .ne. '0' ) THEN
			    IF ( ohdr (itrc,ioh) .eq. 'ALL' ) THEN
				ohdr (itrc,ioh) = crec (k)
			    ELSE
				CALL ST_LSTR ( ohdr (itrc,ioh), ll, ir )
				ohdr (itrc,ioh) =
     +				    ohdr (itrc,ioh)(1:ll) // '-' //
     +				    crec (k)
			    END IF
			END IF
		    END DO
		    CALL ST_LSTR ( ohdr (itrc,ioh), nch, ier )
		    IF ( ohdr(itrc,ioh) (nch:nch) .eq. '/' ) THEN
			ll = nch + 1
			ohdr (itrc,ioh) (ll:) = 'ALL'
		    END IF
		END DO
		CALL SRCH_SET ( itrc, kmch, kase, kpdz, kltp,
     +				idplot, iddfld, icntyp, ivrtyp,
     +				icmtyp, charcn, realcn, ncn, iret )
		IF ( iret .ne. 0 ) THEN
		    CALL SRCH_ERR ( iret, 'SRCH_SET', ier )
		    STOP
		END IF
	    END IF
	END DO
	CLOSE (luni)
C
C*	Loop over sets of files.
C
	ios = 0
	DO WHILE ( ios .eq. 0 )
C
C*	    Read a list of file names.
C
	    nfile = 0
	    DO WHILE ( ios .eq. 0 .and. nfile .lt. NFLS )
		READ (lunl,'(A)',IOSTAT=ios) inrec
		IF ( ios .eq. 0 ) THEN
		    nfile = nfile + 1
		    filnam (nfile) = inrec
		END IF
	    END DO
	    IF ( nfile .gt. 0 )
     +		CALL SRCH_DRV ( filnam, nfile, luni, quiet, itrc,
     +			        iret )
	END DO
	CLOSE (lunl)
C*
	IF ( iret .ne. 0 ) THEN
	    CALL SRCH_ERR ( iret, 'SRCH_DRV', ier )
	    IF ( iret .lt. 0 ) STOP
	END IF
	mtrc = 0
	CALL SRCH_GET ( mtrc, mdpth, mxpts, msrti, mkndx,
     +			trace, mknts, clbls, rlbls, srtyp,
     +			iret )
C
C*
C* ------- Debug here:  	CALL SRCH_DMP ( ier )
C*
C
	IF ( mtrc .ne. ntrc ) THEN
	    WRITE (6,*)  ' Number of traces found does not match ',
     +			 'the number requested.'
	    STOP
	END IF
	DO it = 1, mtrc
	    mdpth = 0
	    mxpts = MXDPH
	    CALL SRCH_GET ( it, mdpth, mxpts, msrti, mkndx,
     +			    trace, mknts, clbls, rlbls, srtyp, ier )
	    IF ( ier .ne. 0 ) THEN
		CALL SRCH_ERR ( ier, 'SRCH_GET', ierr )
		IF ( ier .lt. 0 ) STOP
	    END IF
	    IF ( mdpth .gt. 0 ) THEN
		msrti (1) = 0
		mxpts = NPTMX
		DO id = 1, mdpth
		    CALL SRCH_GET ( it, id, mxpts, msrti, mkndx,
     +				    trace (1,id), mknts, clbls, rlbls,
     +				    srtyp, ier )
		    IF ( id .eq. 1 ) crtype = srtyp
		    IF ( ier .ne. 0 ) THEN
			CALL SRCH_ERR ( ier, 'SRCH_GET', ierr )
			IF ( ier .lt. 0 ) STOP
		    END IF
		END DO
C
C*		Write out this trace data.
C
		IF ( crtype .eq. ' ' .or. crtype .eq. 'n' ) THEN
		    IF ( clbls (1) .ne. ' ' ) THEN
			crtype = 'c'
		    ELSE IF ( rlbls (1) .gt. -1.E30 ) THEN
			crtype = 'r'
		    ELSE
			WRITE (6,*) ' Trace data has no label. '
			STOP
		    END IF
		END IF
		IF ( crtype .ne. 'c' ) THEN
		    WRITE (6,*)  'Labels must be character date-times.'
		    STOP
		END IF
		DO i = 1, 9
		    CALL ST_RMBL ( ohdr (it,i), ohdr (it,i), lhdr (i),
     +				   ier )
		END DO
		DO i = 1, mxpts
		    IF ( chhmm .ne. ' ' ) THEN
			clbls (i) = clbls (i) (1:8) // chhmm //
     +				    clbls (i) (13:)
		    END IF
		    CALL ST_LSTR ( clbls (i), lhdr(4), ier )
		    ohdr (it,4) = clbls (i) (1:10) // clbls (i) (13:)
		    CALL ST_RMBL ( ohdr (it,4), ohdr (it,4), lhdr (4),
     +				   ier )
		    trcout (1) = trace (i,1)
		    DO j = 2, mdpth
			trcout (j) = trace (i,j) / trace (i,1)
		    END DO
		    WRITE (luno, 1111) (ohdr(it,j)(1:lhdr(j)),j=1,9),
     +			(trcout (iq), iq = 1, mdpth )
1111		    FORMAT (9(A,1X),'=',12(1X,E16.9))
		END DO
	    END IF
	END DO
	STOP
	END 
